mdt.in <-
function(...){
	obj <- list(...)
	num.obj <- length(obj)
	type <- vector(length = num.obj)
	for (i in seq(along = type)) type[i] <- class(obj[[i]])
	x <- c("DNAbin", "phylo", "data.frame") %in% unique(type)  
	if (!x[1]) stop("\nNo alignment specified.")
	if (!x[2]) stop("\nNo phylogram specified.")
	if (!x[3]) stop("\nNo constraints specified.")
	
	# Liste erstellen:
	x <- vector(mode = "list", length = 5)
	names(x) <- c("partitions", "sequences", "trees", 		"final.tree", "constraints")
	x$sequences <- obj[type == "DNAbin"]
	x$trees <- obj[type == "phylo"]
	x$constraints <- obj[type == "data.frame"]
	x$constraints <- x$constraints[[1]]
	x$partitions <- length(x$sequences)
	x$final.tree <- x$trees[[1]]
	if (x$partitions != length(x$trees))
		x$trees <- x$trees[-1]
	if (x$partitions < length(x$trees))
		stop("\nMore trees than sequence partitions.")
	if (x$partitions > length(x$trees))
		stop("\nLess trees than sequence partitions.")
		
	for (i in 1:x$partitions){
        test <- rownames(x$sequences[[i]]) %in% x$trees[[i]]			$tip.label
		if (FALSE %in% test){ 
			mismatch <- rownames(x$sequences[[i]])[!test]
			cat(paste("\n", length(mismatch), " taxon names of the sequence alignment of partition ", i,  " do not match any name of tips in the corresponding phylogenetic tree:\n", sep = ""))
			print(mismatch)
			stop("Check your input data!")
		}
		test <- x$trees[[i]]$tip.label %in% rownames(x$sequences			[[i]])	
		if (FALSE %in% test){ 
			mismatch <- x$trees[[i]]$tip.label[!test]
			cat(paste("\n", length(mismatch), " names of the tips of the phylogentic tree assigned to sequence alignment of partition ", i,  " do not match any of its sequence names:\n", sep = ""))
			print(mismatch)
			stop("Check your input data!")
		}		
	}
	colnames(x[[grep("constraints", names(x))]]) <- 		c("bound", "tip1", "tip2", "age")
	
	# Order constraints in decreasing age
	index <- as.numeric(levels(x$constraints$age))		[as.integer(x$constraints$age)]
	index <- order(index, decreasing = TRUE)
	x$constraints <- x$constraints[index,]
	
	# Age contraints in the outgroup?
	num.tips <- length(x$final.tree$tip.label)
	desc.of.root <- x$final.tree$edge[x$final.tree$edge[,1] == 		num.tips + 1, 2]
	# outgroup is the lower most
	outgroup <- desc.of.root[length(desc.of.root)]
    if (outgroup > num.tips)
    	outgroup <- x$final.tree$tip.label[DESCENDANTS(			x$final.tree, node = outgroup)]
    if (any(outgroup %in% unique(c(levels(x$constraints[,2]), 		levels(x$constraints[,2])))))
     	stop("\nSome of your constraints involving outgroup taxa. This is not allowed in multidivtime. Exciting ....")
     
    
	test <- as.vector(unlist(x$constraints[,2:3])) %in% 		x$final.tree$tip.label
	if (!all(test)){
		misspelled <- paste(as.vector(unlist(			x$constraints[,2:3]))	[!test], collapse = " ")
		stop(paste("\nTaxon names in the constraints do not correspond to taxon names in your final phylogenetic tree. The following taxa seem to be misspelled:\n\n", misspelled))
		}
	class(x) <- "mdt.in"
	x
}

