multidivtime <-
function(x, file = NULL, start = "baseml", part = 1, runs = 1,
    path = NULL, transfer.files = TRUE, LogLCheck = 100, 
    plot = TRUE){
	
	
	#	get OS: important to guarantee cross-platform 
	#	compatibility. Quartz() is only know under Mac OS.
	#	What about Linux?
	#____________________________________________________________
	
	OS <- Sys.info()[names(Sys.info()) == "sysname"]
	if (OS == "Linux")
		plot <- FALSE
	
	# set default filename "LAGOUT"
	if (is.null(file)) file <- "LAGOUT"
	
	# 	Check input object x:
	#	- Is 'baseml.ctr' in the working directory? 
	#	- Is 'multicntrl.dat' in the working directory?
	#	- Has a path object been specified?
	#	- Can the four executables be found?
	#	- Is there an 'inseed file'
	#____________________________________________________________
	
	if (!"baseml.ctr" %in% ls(1)) 
		stop("\'baseml.ctr\' is missing. Type in", 			" \'?baseml.ctr\' for more information.")
		
	if (!"multicntrl.dat" %in% ls(1))
		stop("\'multicntrl.dat\' is missing. Type in", 			" \'?multicntrl.dat\' for more information.")
	
	#if (!path %in% ls(1))
		#stop("The argument \'path\' is missing. Please specify", 			" the path to the executables and try again.")
		
	exe <- list.files(path)
	programs <- c("baseml", "paml2modelinf", "estbranches", 		"inseed", "multidivtime")
	if (OS == "Windows"){
		programs[3] <- "estbranches_dna"
		programs[-4] <- paste(programs[-4], "exe", sep = ".")
	}
	exe <- programs %in% exe
	if (!all(exe)){
		error <- paste(programs[!exe], collapse = "\n\t")
		error <- paste("\nThe following executables or files are", 			" missing in the directory that you have specified", 			" via the \'path\' argument:\n\t", error, sep = "")
		stop(error)
	}
	
	# attach input object x	
	attach(x, warn.conflicts = FALSE)
	
	# store working directory
	rwd <- getwd()
	
	cat(paste("\nFound", partitions, 
		"partition(s)."))
	
	start <- match.arg(start, c("baseml", "estbranches", 
        "multidivtime"))
		
	if (start == "baseml"){
		setwd(path)
		for (i in part:x$partitions){
			partition <- x$sequences[[i]]
			ntax <- dim(partition)[[1]]
			nchar <- dim(partition)[[2]]
			taxnames <- rownames(partition)

			#####################################################
			# 1.	export sequence file to PAML
			#####################################################
			
			filename <- paste("testseq.", file, i, sep = "")
			write(c(ntax, nchar), filename, append = FALSE)
			for (j in 1:ntax){
				s <- paste(partition[j,], collapse = "")
				s <- c(taxnames[j], s)
				write(s, filename, append = TRUE)
			}
	
			#####################################################
			# 2.	export tree file(s) to PAML
			#####################################################
			
			filename <- paste(file, i, ".tree", sep = "")
			write(paste(ntax, "1"), filename, append = FALSE)
			x$trees[[i]]$edge.length <- NULL
			x$trees[[i]]$node.label <- NULL
			write.tree(x$trees[[i]], filename, append = TRUE)
	
			#####################################################
			# 3.	write baseml.ctl
			#####################################################
			
			filename <- "baseml.ctl"
			attach(baseml.ctr, warn.conflicts = FALSE)
			write(paste("     seqfile = testseq.", file, i, 				sep = ""), filename)
			write(paste("     outfile = paml", file, i, ".out", 				sep=""), filename, append = TRUE)
			write(paste("    treefile = ", file, i, ".tree\n", 				sep=""), filename, append = TRUE)
			write(paste("       noisy = ", noisy, 
				"    * 0,1,2,3: how much rubbish on the screen", 				sep=""), filename, append=TRUE)
   			write(paste("     verbose = ", verbose, 
   				"    * 1: detailed output, 0: concise output", 				sep=""), filename, append=TRUE)
   			write(paste("     runmode = ", runmode, 
   				"    * 0: user tree;  1: semi-automatic;  ", 				"2: automatic", sep = ""), filename, append=TRUE)
    		write("                         * 3: StepwiseAddition; (4,5):PerturbationNNI", filename, append = TRUE)
    		write(paste("       model = ", model, "    * 0:JC69, 1:K80, 2:F81, 3:F84, 4:HKY85", sep=""), filename, append = TRUE)
    		write("                         * 5:T92, 6:TN93, 7:REV, 8:UNREST, 9:REVu; 10:UNRESTu", filename, append = TRUE)
    		write(paste("       Mgene = ", Mgene, "    * 0:rates, 1:separate; 2:diff pi, 3:diff kapa, 4:all diff", sep=""), filename, append=TRUE)
    		write(paste("   fix_kappa = ", fix_kappa, "    * 0: estimate kappa; 1: fix kappa at value below", sep=""), filename, append=TRUE)
    		write(paste("       kappa = ", kappa, "    * initial or fixed kappa", sep=""), filename, append=TRUE)
    		write(paste("   fix_alpha = ", fix_alpha, "    * 0: estimate alpha; 1: fix alpha at value below", sep=""), filename, append=TRUE)
			write(paste("       alpha = ", alpha, "    * initial or fixed alpha, 0:infinity (constant rate)", sep=""), filename, append=TRUE)
			write(paste("      Malpha = ", Malpha, "    * 1: different alphas for genes, 0: one alpha", sep=""), filename, append=TRUE)
    		write(paste("       ncatG = ", ncatG, "    * # of categories in the dG, AdG, or nparK models of rates", sep=""), filename, append=TRUE)
    		write(paste("     fix_rho = ", fix_rho, sep=""), filename, append=TRUE)  
    		write(paste("         rho = ", rho, "    * initial or given rho,   0:no correlation", sep = ""), filename, append = TRUE)
    		write(paste("       nparK = ", nparK , "    * rate-class models. 1:rK, 2:rK&fK, 3:rK&MK(1/K), 4:rK&MK", sep=""), filename, append=TRUE)
    		write(paste("       clock = ", clock, "    * 0:no clock, 1:clock; 2:local clock; 3:CombinedAnalysis", sep=""), filename, append=TRUE)
    		write(paste("       nhomo = ", nhomo, "    * 0 & 1: homogeneous, 2: kappa for branches, 3: N1, 4: N2", sep=""), filename, append=TRUE)
  			write(paste("       getSE = ", getSE, "    * 0: do not want them, 1: want S.E.s of estimates", sep=""), filename, append=TRUE)
 			write(paste("RateAncestor = ", RateAncestor, "    * (0,1,2): rates (alpha>0) or ancestral states", sep=""), filename, append=TRUE)
   			write(paste("  Small_Diff = ", Small_Diff, sep=""), filename, append=TRUE)
			write(paste("   cleandata = ", cleandata, "    * remove sites with ambiguity data (1:yes, 0:no)?", sep=""), filename, append=TRUE)
			write("*            ndata = 5", filename, append=TRUE)
			write("*            icode = 0    * (with RateAncestor=1. try GC in data,model=4,Mgene=4)", filename, append = TRUE)
			write("*      fix_blength = -1    * 0: ignore, -1: random, 1: initial, 2: fixed", filename, append=TRUE)
			write(paste("      method = ", method, "    * 0: simultaneous; 1: one branch at a time", sep=""), 			 filename, append = TRUE)
			detach(baseml.ctr)
			
			#####################################################
			# 5.	run baseml 
			#####################################################
			
			cat(paste("\n\nrunning BASEML on partition", i, 
				"... "))
			bml.start <- Sys.time()
			outfile <- paste("o", file, i, sep = "")
			exe <- paste(".", programs[1], sep = "/")
			system(paste(exe, "baseml.ctl >", outfile))
			bml.stop <- Sys.time()
			time.used <- round(bml.stop - bml.start, digits = 2)
			if (!file.exists(outfile) | 
				file.info(outfile)$size == 0)
				stop(" ... failed.", call. = FALSE)
			opaml <- scan(outfile, what = "c", sep = "\n", 				quiet = TRUE)
			exc <- grep("Time used", opaml)
			if (length(exc) == 0) 
				stop("failed.\nCheck the output file \'", outfile, 
					"\'", sep = "", call. = FALSE)
			cat(" finished.\n-- Time used: ", format(time.used))
		}
	} # END: start == "baseml"
	
	if (start == "baseml" || start == "estbranches"){
		setwd(path)	
		for (i in part:x$partitions){
			partition <- x$sequences[[i]]
			ntax <- dim(partition)[[1]]
			nchar <- dim(partition)[[2]]
			taxnames <- rownames(partition)
	
			#####################################################
			# 7.	export sequence file to multidivtime
			#####################################################
			filename <- "testseq"
			write(c(ntax, nchar), filename, append = FALSE)
			for (j in 1:ntax){
				s <- paste(partition[j,], collapse = "")
				s <- c(taxnames[j], s)
				write(s, filename, append = TRUE)
			}
		
			#####################################################
			# 8. 	export tree file to multidivtime
			#####################################################
			if (is.null(x$final.tree$root.edge))
				x$final.tree$root.edge <- 0
			ntax <- length(x$final.tree$tip.label)
			filename <- paste(file, ".tree", sep = "")
			write(paste(ntax, "1"), filename, append = FALSE)
			x$final.tree$edge.length <- NULL
			x$final.tree$node.label <- NULL
			x$final.tree$root.edge <- NULL
			write.tree(x$final.tree, filename, digits = 0, 				append = TRUE)
	
			#####################################################
			# 9. run paml2modelinf
			#####################################################
			setwd(path)
			cat("\n\nrunning PAML2MODELINF ...")
			exe <- paste(".", programs[2], sep = "/")
			system(paste(exe, " paml", file, i, 
				".out modelinf.", file, i, sep = ""))
			cat(" finished.")
			
			#####################################################
			#	10. write hmmcntrl.dat
			#####################################################	
			filename <- "hmmcntrl.dat"
            h <- c("/* Which Model to use? */",
                paste("modelinf.", file, i, sep=""),
                paste("L  /* How much output? Options: L = Loud", 					"mode (prints more output, the"),
    			paste("      default), Q = Quiet mode (prints", 
    			    "less output - use with parametric"),
                "      bootstrap) */", 
                paste("D  /* Predict Secondary Structure?",
                    "Options: P= predict, D = do not predict"),
                "      (the default option) */",
                paste("N  /* Does user tree specify names (N) or",
                    "specify order (O) of sequences"), 
    			"      in sequence data file? */",
    		"   /* The topology is in the file listed below */",
    			paste(file,".tree", sep = ""),
    			"   /* End of hmmcntrl.dat */"
    		)
    		write(h, filename)
						
			#####################################################
			# 11. 	run estbranches
			#####################################################
			
			cat(paste("\n\nrunning ESTBRANCHES on partition", i, 
				"... "))
			est.start <- Sys.time()
			exe <- paste(".", programs[3], sep = "/")
			system(paste(exe, " oest.", file, i, 
				" > out.oest.", file, i, sep = ""))
			est.stop <- Sys.time()
			time.used <- round(est.stop - est.start, digits = 2)
			est.res <- scan(paste("out.oest.", 					file, i, sep=""), what = "c", quiet = TRUE)
			if (length(est.res) > 0){
				cat(" finished.")
				cat("\n-- Time used: ", format(time.used))
			}
			else { 
				cat("\n \n")
				stop("\nESTBRANCHES failed. Check your data!", 					call. = TRUE)
			}
			
		}
	
		#####################################################
		# 12. 	check likelihood optimization
		#####################################################
		
		cat("\n\nCHECK PERFORMANCE OF LIKELIHOOD OPTIMIZATION:")
		for (i in 1:x$partitions){
			cat(paste("\n\nPartition ", i, ":", sep=""))
			L.baseml <- scan(paste("paml", file, i, ".out", 
				sep = ""), what="character", quiet = TRUE)
			x <- grep("lnL", L.baseml)
			L.baseml <- L.baseml[x:(x+10)]
			y <- grep("):", L.baseml)
			L.baseml <- as.numeric(L.baseml[y+1])
	
			L.estbran <- scan(paste("oest.", file, i, sep = ""), 				what="character", quiet = TRUE)
			z <- grep("FINAL", L.estbran) - 3 
			L.estbran <- as.numeric(L.estbran[z])
			cat(paste("\nlnL (baseml)      =", L.baseml))
			cat(paste("\nlnL (estbranches) =", L.estbran))
			p <- abs(L.baseml - L.estbran)
			if (p > LogLCheck) 
				stop(paste("\nThe difference in LogLik values ", 					"exceeds threshold of ", LogLCheck, 
					". Exiting ...", sep = ""))
			else cat(paste("\nThe difference in LogLik values ", 				"does not exceed threshold of ", LogLCheck, 
				". Proceeding ...", sep = ""))
		}
	}
	
# >>> START == "MULTIDIVTIME"	
	
	#####################################################
	# 	13. 	translate nodes and fix age constraints
	#####################################################
	setwd(path)
	cons <- constraints
	X <- scan(file = paste("oest.", file, "1", sep = ""), 
		what = "character", quiet = TRUE)
	semikolon <- grep(";", X) # end of tree
	TREE <- paste(X[1:semikolon], collapse = "")
	TREE <- read.tree(text=TREE)
	# identify nodes ...
	nb.tips <- length(TREE$tip.label)
	nb.nodes <- TREE$Nnode
	nb.cons <- dim(cons)[1]
	CC <- cbind(TREE$tip.label, 1:nb.tips)
	R.nodes <- NULL
	for (i in seq(along = cons[,1])){
		taxon1 <- as.numeric(CC[,2][CC[,1] == 				as.character(cons[i,2])])
		taxon2 <- as.numeric(CC[,2][CC[,1] == 
			as.character(cons[i,3])])
		node <- getMRCA(TREE, c(taxon1, taxon2))
		R.nodes <- c(R.nodes,node)
	}
	# ... and translate them
	cat("\n\nTranslating internal node numbers ...")
	mdt.nodes <- (nb.tips+nb.nodes-1):nb.tips
	r.nodes <- (nb.tips+1):(nb.tips+nb.nodes)
	node.table <- data.frame(mdt.nodes, r.nodes)
	cat(" finished.")
	 
	# fix age constraints
	cat("\n\nSetting age constraints ...")
	cons <- data.frame(cbind(as.character(cons$bound), R.nodes, 		as.character(cons$age)))
	cons.R <- cons # store R constraints
	MDT.nodes <- NULL
	for (i in seq(along = cons[,1])) MDT.nodes <- c(MDT.nodes, 		node.table[,1][node.table[,2] == cons[i,2]])
	cons <- data.frame(cbind(as.character(cons$V1), MDT.nodes, 		as.character(cons$V3)))
	# print control trees
	if (plot){
		n <- length(TREE$tip.label)-1
		if (OS == "Darwin")
			quartz(width = 10, height = 6)
		if (OS == "Windows")
			x11(width = 10, height = 6, 1)
		par(mfrow=c(1,2))
		plot(TREE, y.lim = nb.tips)
		nodelabels()
		title("Node assignment in R")
		plot(TREE, y.lim = nb.tips)
		mdt.nodes <- (nb.tips+nb.nodes-1):nb.tips
		nodelabels(mdt.nodes)
		title("Node assignment in Multidivtime")
	
		circle <- rep(NA, nb.nodes)
		x <- as.numeric(as.character(cons.R[,2]))-nb.tips
		circle[x] <- 21
		age <- rep(NA, nb.nodes)
		for (i in seq(along = cons[,1])){
			y <- as.numeric(as.character(cons.R[i,2]))-nb.tips
			age[y] <- as.numeric(as.character(cons.R[i,3]))
		}
	
		if (OS == "Darwin")
			quartz(width = 10, height = 6)
		if (OS == "Windows")
			x11(width = 10, height = 6, 1)
		plot(TREE, use.edge.length=FALSE, y.lim=nb.tips)
		nodelabels(pch=circle, cex=3.5, frame="n", col="orange", 			bg = "orange")
		nodelabels(age, cex=0.8, frame="n")
		title("Age constraints")
	}
	cat(" finished.")
	
   	#####################################################
	#	15. 	write multicntrl.dat
	#####################################################
	
	# attach 'multicntrl.dat'
	attach(multicntrl.dat, warn.conflicts = FALSE)
	
	# set 'rtrate' and 'rtratesd': If a number is given,
	# the number is used, else they are estimated by the 
	# 	> median of the branch lengths divided by rttm <
	#____________________________________________________________
	
	if (rtrate == "median" | rtratesd == "median") {
		cat("\n\nCalculating median of branch lengths ('amount", 			"of evolution') \nfor the root rate prior ...")
		xx <- numeric(nb.tips + nb.nodes)
    	for (i in 1:dim(TREE$edge)[1]) xx[TREE$edge[i, 2]] <- 
    		xx[TREE$edge[i,1]] + TREE$edge.length[i]
    	MEDIAN <- median(xx[1:nb.tips])
    	cat(" finished.")
    	if (rtrate == "median"){
    		rtrate <- MEDIAN/rttm
    		cat("\n'rtrate' set to ", rtrate, ".", sep = "")
    	}
    		
    	if (rtratesd == "median"){
    		rtratesd <- MEDIAN/rttm
    		cat("\n'rtratesd' set to ", rtratesd, ".", sep = "")
    	}
    }
	
	m1 <- vector(length = 5)
	m1[1] <- paste("/* the following lines are all needed in", 		" multicntrl.dat ...")
	m1[2] <- paste("   do not add or delete lines but", 		"change entry on left of each")
	m1[3] <- "   line as you see fit ...  */"
	m1[4] <- paste(file,".tree", sep = "")
	m1[5] <- paste(partitions, "... number of genes ...", 		"FOLLOWING LINES CONTAIN ONLY NAMES OF DATA FILES")
	
	m2 <- vector(length = partitions)
	for (i in 1:partitions) 
		m2[i] <- paste("oest.", file, i, sep = "")
		
	m3 <- vector(length = 21)
	m3[1] <- paste(numsamps, "... numsamps: How many times", 		"should the Markov chain be sampled?")
	m3[2] <- paste(sampfreq, "... sampfreq: How many cycles", 		"between samples of the Markov chain?")	
	m3[3] <- paste(burnin, "... burnin: How many cycles before", 		"the first sample of Markov chain?")	
	m3[4] <- paste(rttm, "... rttm: a priori expected number", 		"of time units between tip and root")
	m3[5] <- paste(rttmsd, "...  rttmsd: standard deviation of", 		"prior for time between tip and root")
	m3[6] <- paste(rtrate, "... rtrate: mean of prior", 		"distribution for rate at root node")	
	m3[7] <- paste(rtratesd, "... rtratesd: standard", 		"deviation of prior for rate at root node")
	m3[8] <- paste(brownmean, "... brownmean: mean of prior", 		"for brownian motion constant 'nu'")
	m3[9] <- paste(brownsd, "... brownsd: std. deviation of", 		"prior for brownian motion constant 'nu'")
	m3[10] <- paste("/* the following lines are all needed", 		"(i.e., do not delete them) but you may ")
	m3[11] <- paste("   not want to alter entries unless you", 		"are familiar with the computer code */")
	m3[12] <- paste(minab, "... minab: parameter for beta prior", 		"on proportional node depth")
	m3[13] <- paste(newk, "... newk: parameter in Markov chain", 		"proposal step")
	m3[14] <- paste(othk, "... othk: parameter in Markov chain", 		"proposal step")
	m3[15] <- paste(thek, "... thek: parameter in Markov chain", 		"proposal step")
	m3[16] <- paste(bigtime, "... bigtime: number higher than", 		"time units between tip and root could")
	m3[17] <- "                   be in your wildest imagination"
	m3[18] <- paste("/* the program will expect the entry below", 		"to be the number of constraints")
	m3[19] <- paste("   and then the specified number of", 		"constraints should follow on")
	m3[20] <- "   subsequent lines */"
	m3[21] <- paste(length(cons[,1]), 					"... number of constraints on node times")
	
	m4 <- vector(length = nb.cons)
	for (i in 1:nb.cons)
		m4[i] <- paste(cons[i,1], cons[i,2], cons[i,3])

	m5 <- vector(length = 3)
	m5[1] <- paste(tipsnotcoll, 						"... number of tips which are not collected at time 0")
	m5[2] <- paste(nodata, "... nodata: 1 means approximate", 		"prior, 0 means approximate posterior")
	m5[3] <- paste(commonbrown, "... commonbrown: 1 if all", 		"genes have same tendency to change rate, 0 otherwise")
	m <- c(m1, m2, m3, m4, m5)
	write(m, file = "multicntrl.dat")
	detach(multicntrl.dat)
	
	#####################################################
	# 16. 	run multidivtime
	#####################################################
	prefix <- c("node", "out", "ratio", "samp", "tree")
	runfile <- paste(file, "_RUN", 1:runs, sep = "")
	
	# delete multidivtime output files of previous runs
	# with identical names (hint by E. Paradis)
	# -----------------------------------------
	pmo <- list.files(path, pattern = paste(prefix, 		collapse = "|"), full.names = TRUE)
	amo <- sapply(runfile, function(prefix, runfile) 		paste(prefix, runfile, sep = "."), prefix = prefix)
	amo <- paste(path, amo, sep = "/")
	file.remove(amo[amo %in% pmo])
	
	# iterate over number of runs
	# ---------------------------
	for (i in 1:runs){
		cat("\n\nrunning number", i, "of", runs, 			"MULTIDIVTIME runs ... ")
		exe <- paste(".", programs[5], sep = "/")
		mdt.start <- Sys.time()
		system(paste(exe, " ", runfile[i], " > out.", 			runfile[i], sep = ""))
		mdt.stop <- Sys.time()
		time.used <- round(mdt.stop - mdt.start, digits = 2)
		check <- scan(paste("out.", runfile[i], sep = ""), 			what = "raw", quiet = TRUE)
		if (length(check) == 0)
			stop("failed")
		else {
			ind <- grep("PROBLEM", check)
			if (length(ind) == 0)
				cat ("finished.")
			else {
				ERROR <- paste(check[ind:length(check)], 					collapse = " ")
				ERROR <- paste("\n\n", ERROR, sep = "")
				stop(ERROR, call. = FALSE)	
			}
		}
		cat("\n-- Time used: ", format(time.used))
	}
	
 	#####################################################
  	# 17.	transfer of files
  	#####################################################
if (transfer.files){
		setwd(rwd)
		order.name <- unlist(strsplit(date(), split = " "))
		order.name <- paste(order.name[c(5, 2, 3, 4)], 			collapse = "-")
		order.name <- paste(file, order.name, sep = "_")
		dir.create(order.name)
		PATH <- paste(rwd, order.name, sep = "/")
		setwd(path)
		
		FILES <- NULL
		for (i in 1:partitions){
			FILES <- c(FILES, 									paste("paml", file, i, ".out", sep = ""))
			FILES <- c(FILES, 									paste("modelinf.", file, i, sep = ""))
			FILES <- c(FILES, 									paste("out.oest.", file, i, sep = ""))
			FILES <- c(FILES, 									paste("oest.", file, i, sep = ""))
		}
		FILES <- c(FILES, paste(file, ".tree", sep = ""))
		
		# REMOVE ?
		for (i in 1:runs){
			FILES <- c(FILES,									paste("tree.", runfile[i], sep = "")) 
			FILES <- c(FILES, 									paste("samp.", runfile[i], sep = ""))
			FILES <- c(FILES, 									paste("ratio.", runfile[i], sep = ""))
			FILES <- c(FILES, 									paste("node.", runfile[i], sep = ""))
			FILES <- c(FILES, 									paste("out.", runfile[i], sep = ""))
		}
		FILES <- c(FILES, "multicntrl.dat")
		
		for (j in seq(along = FILES))
			file.copy(FILES[j], PATH)
		
		cat(paste("\n\nOutput files printed to", PATH, "\n\n\n"))
	}
	
	#####################################################
	# 18. 	read in results (Code due to N.Cusimano)
	#####################################################
	
	setwd(PATH)
	if (runs == 1) mdt <- read.mdt(file, run = 1)
	else {
		mdt <- list(length = runs)
		y <- vector()
		for (i in 1:runs){
			#mdt[[i]] <- read.mdt(runfile[i])
			mdt[[i]] <- read.mdt(file, run = i)
			y <- rbind(y, mdt[[i]]$TABLE[,2])
		}
		colnames(y) <- 1:dim(y)[2]
		y <- y[, which(y[1,] > 0)]
		barplot(y, beside = TRUE, names.arg = colnames(y), 			xlab = "Nodes", ylab = "Age", 					main = "Convergence of Markov Chain monte carlo runs", 			col = heat.colors(runs))	
	}
	detach(x)
	setwd(rwd)

	mdt
}

