`read.mdt` <-
function(name, multicntrl = TRUE, run = 1) { 
	
	# get tree:
	crg <- read.tree(paste("tree.", name, "_RUN", run, sep = ""))
	# get MDT Table
	tab <- scan(file = paste("out.", name, "_RUN", run, sep = ""), 		what = "char", quiet = TRUE, sep = "\n")
	tab <- as.list(tab[grep("Actual",tab)])
	tab <- gsub("Actual time node   ", "", tab,  fixed = TRUE)
	tab <- gsub("Actual time node  ", "", tab,  fixed = TRUE)
	tab <- gsub("Actual time node ", "", tab,  fixed = TRUE)
	tab <- gsub("S.D.", "", tab,  fixed = TRUE)
	tab <- gsub("=", "", tab,  fixed = TRUE)
	tab <- gsub("    ", " ", tab, fixed = TRUE)
	tab <- gsub("   ", " ", tab, fixed = TRUE)
	tab <- gsub("  ", " ", tab, fixed = TRUE)
	tab <- gsub("( ", "", tab, fixed = TRUE)
	tab <- gsub(")", "", tab, fixed = TRUE)
	tab <- gsub("(", "", tab, fixed = TRUE)
	tab <- gsub(",", "", tab,  fixed = TRUE)
	tab <- strsplit(tab, split = " ")
	tab <- unlist(tab)
	tab <- as.numeric(tab)
	tab <- matrix(tab, ncol = 5, byrow = TRUE)
	
	
	colnames(tab) <- c("node.mdt", "age", "S.D.", "min", "max")

  	L <- length(which(tab[,2] == 0))
	N <- length(which(tab[,2] != 0))
		
	node.ape <- c(L:1, (L+N):(L+1))
		       	
    tab <- cbind(tab, node.ape)
       	
	# constraints and priors from multicntrl.dat
	if (multicntrl){
		a <- scan("multicntrl.dat", what = "char", quiet = TRUE)
		
		# get constraints
		cons <- grep("constraints", a)[3] + 4
		coll <- grep("collected", a) - 9

		a1 <-a[cons:coll]

		U <- grep("U", a1)
		L <- grep("L", a1)

		nodes <- as.integer(c(a1[U+1], a1[L+1]))
		ages <- c(a1[U+2], a1[L+2])
		type <- c(rep("U", length(U)), rep("L", length(L)))
		l <- data.frame(rbind(type, nodes, ages), 			stringsAsFactors = FALSE)
		
		#get priors
		rtt <- as.numeric(a[grep("rttm:", a) -2])
		rtt <- c(rtt,as.numeric(a[grep("rttmsd:", a) -2]))
		
		brown <- as.numeric(a[grep("brownmean:", a) -2])
		brown <- c(brown,as.numeric(a[grep("brownmean:", a) -2]))
		
		rtrate <- as.numeric(a[grep("rtrate:", a) -2])
		rtrate <- c(round(rtrate, digits=4),round(as.numeric(a[grep("rtratesd:", a) -2]), digits=4))
		
		p <- data.frame(rbind(rtt,rtrate, brown))
		names(p)<-c("Mean", "SD")
	}      	
	else l <- NULL

    ### build object of class mdt
	out <- list(crg, tab, l, p)
	names(out) <- c("TREE", "TABLE", "CONSTRAINTS", "PRIORS")
	class(out) <- "mdt"
	out
}

