\name{plot.mdt}
\alias{plot.mdt}
\title{Plot chronogram}

\description{
This function takes an object of class \code{mdt} as input and plots a chronogram.
 }
 
\usage{
plot.mdt(mdt, nodes = "all", ages = "SD", bars = "SD", 
	nodes.mdt = FALSE, digits = 0, barcol = "blue", barwd = 2, 
	agecex = 0.7, agefont = 2, agecol = "black", agebg = "#FF4000FF", 
	ageframe = "r", ageadj = 0.5, label = " Mya", 
	points = TRUE, x.lim = FALSE, cex = 1, edge.color = "black", 
	edge.lty = 1, tip.color = "black",  ...)
}

\arguments{
	
  \item{mdt}{list of class \code{mdt}.}
  
  \item{nodes}{vector of mode either character or numeric. \code{nodes = "all"} (the default) plots specified items (ages, bars) on all nodes of the chronogram, \code{nodes = "allButCons"} plots specified items on all nodes of the chronogram, except for the constrained nodes, if \code{nodes} is a numeric vector, ages and/or bars are plotted only on the specified node. (Node numbers can be according to \code{ape} node numbering, but also to MULTDIVTIME node numbering, then \code{mdt.nodes = TRUE}.)}
  
  \item{ages}{vector of mode characer or logical. If \code{ages="SD"} (the default) it plots the ages and the standard deviation on nodes, if \code{ages = "mean"} it plots just the mean node ages onto the nodes if \code{ages = FALSE} no numbers are plotted.}
  
  \item{bars}{vector of mode characer or logical.If \code{bars="SD"} (the default) bar lengths reflect the standard deviation of node ages, if \code{bars = "CI"} bar lengths reflect the 95\% confidence interval of node ages, if \code{ages = FALSE} no bars are plottet.}
  
  \item{nodes.mdt}{logical, if \code{TRUE} any integers specified for argument \code{nodes} are interpreted as node numbers as used in MULTIDIVTIME, else they are interpreted as \code{ape} node numbers. If \code{nodes = "all"} or \code{nodes = "allButCons"}, changing settings of \code{modes.mdt} will not have any effect.}
  
  \item{digits}{numeric, number of digits to which the standard deviation is rounded before plotting.}
  
  \item{barcol}{vector of mode character indicating which color(s) should be used for the error bars.}
  
  \item{barwidth}{integer giving the width of the error bars.}
  
  \item{agecex}{numeric, controlling size of the ages.}
  
  \item{agefont}{an integer specifying the font type of ages: 1 (plain text), 2 (bold, the default), 3 (italic), or 4 (bold italic).}
  
 \item{agecol}{vector of mode character indicating which font color(s) should be used for the ages.}
 
 \item{agebg}{vector of mode character indicating which background color(s) should be used for the ages.}
 
 \item{ageframe}{a character string specifying the kind of frame to be printed around the ages; must be one of "rect" (the default), "circle", "none", or any unambiguous abbreviation of these.}
 
  \item{ageadj}{numeric, controlling position of ages, the default plots it centered on the nodes, e.g., c(0.5,-0.5) plots it above the node and c(0.5,1.5) below the node.}
  
  \item{label}{vector of mode character, specifying what text should be printend right to the time scale. If \code{NA} no text will be plotted, if \code{FALSE} no time scale at all will be plotted}
  
  \item{points}{logical indicating if a small open circle should be plotted onto the nodes. The authors do not really agree on the usefulness of this argument. NC thinks it makes the plot clearer, CH is not so sure if this is true. Make your own choice ...}
  
  \item{x.lim}{vector of mode numeric or logical. If \code{FALSE}, the time scale (and thereby the whole plot) ends at the mean root node age. This might result in the error bar for the root age being cut. In this case, \code{x.lim = TRUE} will extend time scale down to the maximum value of the error bar. Besides, \code{x.lim} can be used like the correspoding argument of \code{\link{plot.phylo}}.}
  
  \item{cex}{numeric, \code{cex} of \code{\link{plot.phylo}}.}
  
  \item{edge.color}{vector of mode character, \code{edge.color} of \code{\link{plot.phylo}}.}
  
  \item{tip.color}{vector of mode character, \code{tip.color} of \code{\link{plot.phylo}}.}
  
  \item{edge.lty}{integer, \code{edge.lty} of \code{\link{plot.phylo}}.}
  
  \item{\dots}{Further arguments to be given to \code{\link{plot.phylo}}.}
}

\details{
Use the arguments \code{ages} and \code{bars} to plot ages on nodes either alone or with standard deviation and/or bars with length corresponding to standard deviation or 95\% confidence interval. All possibilities can be combined.
}


\note{
 Sometimes root node labels are plotted at the bottom of the plot. Replotting normally corrects it.
}

\author{Natalie Cusimano, Christoph Heibl}

\seealso{\code{\link{read.mdt}}, \code{\link{plot.cons}}, \code{\link{plot.phylo}}}

\examples{
	
# Load example object of class 'mdt'.
data(mdt.example)

# The default plotting pattern ...
plot.mdt(mdt.example)

# Three examples to demostrate the flexibility of 'plot.mdt':
# Show mean ages on the nodes and SD as error bars ...
plot.mdt(mdt.example, ages = "mean", ageframe = "c", agebg = 
	"purple", agecol = "yellow", bars = "SD", barwd = 3)

# The same, but mean ages without a frame and above the nodes ...
plot.mdt(mdt.example, ages = "mean", ageframe = "n", agecol = 
	"darkred", bars = "SD", barwd = 3, agead = c(0.5,-0.5))

# Show only bars representing 95-percent confidence intervals ...
plot.mdt(mdt.example, ages = FALSE, bars = "CI", barwd = 4, 
	barcol = "green")
}