\name{read.mdt}
\alias{read.mdt}
\title{Create mdt object from multidivtime output}
\description{
This functions reads the multidivtime output and creates an object of class \code{mdt} that makes visualization and analysis of the results straightforward.
}
\usage{
read.mdt(name, multicntrl = TRUE, run = 1)
}
\arguments{
  \item{name}{vector of mode \code{character} giving the name of the analysis (i.e. the same string that to which you have set the \code{file} argument when calling \code{\link{multidivtime}}; defaults to \code{file = "LAGOUT"})}
  \item{multicntrl}{logical indicating if the contraints should also be read in from the multicntrl.dat file}
  \item{run}{integer, from which run to read the \code{multidivtime} output}
}
\details{
If you want to use \code{read.mdt} on a MULTIDIVTIME analysis, that as not been carried out by \code{LAGOPUS} but 'by hand', make sure that you store these three files in the working directory:
\tabular{ll}{
(1)\tab multicntrl.dat	\cr
(2)\tab tree.<your_analysis_name>	\cr
(3)\tab out.<your_analysis_name>	\cr
}
Then issue something like
\code{x <- read.mdt(your_analysis_name, multicntrl = TRUE)}.
}

\value{
 list of class \code{mdt} containing the following components:
 
  \item{TREE}{object of class \code{"phylo"}, i.e. the chronogram (from file tree.<your_analysis_name>).}
  
  \item{TABLE}{containing estimates for mean node age, standard deviation of node age, and maximum and minimum value of the 95 \% confidence interval. The first column refers to the node numbers as used by \code{multidivtime}; the last column gives their correspnding translation to \code{ape} node numbers (from file out.<your_analysis_name>).}
  
  \item{CONSTRAINTS}{data frame containing  information about the constraints used for analysis (from file \code{\link{multicntrl.dat}}).}
  
   \item{PRIORS}{data frame containing  information about the three priors (rtt: prior for time between tip and root; rtrate: prior for rate at root node; brown: prior for Brownian motion constant 'nu') used for analysis (from file \code{\link{multicntrl.dat}}).}
   
}

\author{Natalie Cusimano (\email{cusimano@bio.lmu.de})}

\seealso{\code{\link{multidivtime}}, \code{\link{plot.mdt}}}